//  
//  2  10.04.2014

function MainMech(canvas) {

    //  

    var context = canvas.getContext("2d");                  //  context  
    document.oncontextmenu=function(e){return false};       //   

    // ***    ***

    const fps = 5;             	    // frames per second -    

    //  

    const w = canvas.width;		    //  
    const h = canvas.height;		//  
    const n = 50;                   //     (,   )
    const m = 50;                   //     (,   )
    const cellW = w/n;              //  
    const cellH = h/m;              //  

    var pause = true;
    var intervalID;                 //    

    // "" : [B = 000100000, L = 001100000]
    const B = "000100000";            //  
    const L = "001100000";            //  

    //   

    var mouseX;     var mouseY;                 //   

    canvas.onmousedown = function(e){           //     
        var life;
        if (e.which == 1) life = true;          //       
        else if (e.which == 3) life = false;    //       
        else return;

        setCell(e, life);
        canvas.onmousemove = function(e) {setCell(e, life);};   // ,     
    };

    document.onmouseup = function(e){           //     
        canvas.onmousemove = null;              //    -   
    };

    function refreshMouseCoords(e){             //      mouseX  mouseY
        var rect = canvas.getBoundingClientRect();
        mouseX = e.clientX - rect.left;
        mouseY = e.clientY - rect.top;
    }


    //   

    var cells;                      //  
    var cellsBuf = [];              //     
    for (var i = 0; i < n; i++) cellsBuf[i] = [];
    function generateRandomField(n, m) {        //      /
        cells = [];
        for (var i = 0; i < n; i++) {
            cells[i] = [];
            for (var j = 0; j < m; j++) {
                cells[i][j] = (Math.random() >= 0.5);
            }
        }
    }

    function setCell(e, life){                  //        
        refreshMouseCoords(e);                  //     mouseX, mouseY
        if (mouseX < 0 || mouseX >= w || mouseY < 0 || mouseY >= h) return;         //    
        var i = Math.floor(mouseX/cellW);       //    
        var j = Math.floor(mouseY/cellH);       //    
        if (cells[i][j] != life) {
            cells[i][j] = life;
            draw();
        }
    }

    //   

    function step() {
        tick();
        draw();
    }

    MainMech.prototype.changePauseState = function() {          //  
        if (!pause) stopSystem();
        else startSystem()
    };

    MainMech.prototype.nextStep = function(){                   //  " "
        stopSystem();
        step();
    };

    function startSystem() {
        pause = false;
        intervalID = setInterval(step, 1000/fps);
        document.getElementById('pause').value = "Stop";
    }

    function stopSystem() {
        pause = true;
        clearInterval(intervalID);
        document.getElementById('pause').value = "Start";
    }

    //   

    // ,   
    function next(i, n) {if (i == (n-1)) return 0; else return i+1;}
    function prev(i, n) {if (i == 0) return n-1; else return i-1;}

    function tick(){                             // ,     
        //   cells  cellsBuf
        for (var i0 = 0; i0 < n; i0++)
            for (var j0 = 0; j0 < m; j0++)
                cellsBuf[i0][j0] = cells[i0][j0];

        for (var i = 0; i < n; i++) {
            for (var j = 0; j < m; j++) {

                //       
                var near = 0;
                if (cellsBuf[prev(i, n)] [prev(j, m)])   near++;
                if (cellsBuf[prev(i, n)] [j])            near++;
                if (cellsBuf[prev(i, n)] [next(j, m)])   near++;
                if (cellsBuf[i]          [prev(j, m)])   near++;
                if (cellsBuf[i]          [next(j, m)])   near++;
                if (cellsBuf[next(i, n)] [prev(j, m)])   near++;
                if (cellsBuf[next(i, n)] [j])            near++;
                if (cellsBuf[next(i, n)] [next(j, m)])   near++;

                if (cellsBuf[i][j])                     //   
                    cells[i][j] = (L[near] == '1');     //      L
                else                                    //   
                    cells[i][j] = (B[near] == '1');     //      B

            }
        }
    }

    // 

    function draw(){
        context.clearRect(0, 0, w, h);          //  
        for (var i = 0; i < n; i++){
            for (var j = 0; j < m; j++){
                if (cells[i][j]){
                    context.beginPath();
                    context.rect(i*cellW, j*cellH, cellW, cellH);
                    context.closePath();
                    context.fill();
                }
            }
        }
    }

    //  
    generateRandomField(n, m);                  //  
    startSystem();

}